!  ==========
!--MAR_AO.inc
!  ==========


!---fields given by ocean/ice model----------------------
! sicsAO: sea ice fraction (from 0 to 1)
! hicAO: sea ice thickness
! hsnoAO: snow thickness over sea ice
! albAO: weighted albedo on sea-ice (2) or on open-water (1)
! UoceAO: surface ocean velocity along X-axis
! VoceAO: surface ocean velocity along Y-axis
! UiceAO: surface ice velocity along X-axis
! ViceAO: surface ice velocity along Y-axis
! srftAO: surf. temperature (K) 

	real*8   sicsAO(mx,my),hicAO(mx,my),hsnoAO(mx,my)
	real*8   UoceAO(mx,my),VoceAO(mx,my)
	real*8   UiceAO(mx,my),ViceAO(mx,my)
	real*8   srftAO(mx,my,mw),albAO(mx,my,mw)
	common /ocefld/sicsAO,albAO,srftAO,hicAO,hsnoAO,UoceAO
        common /ocefld2/VoceAO,UiceAO,ViceAO

       integer aoss,aogla,aoalb,aotic,aohic,aohsn,ao_uo,ao_vo
       integer ao_ui,ao_vi !if gt 0 then MAR has receveid the field => update of the value
       common /aoflag1/aoss,aogla,aoalb,aotic,aohic,aohsn
       common /aoflag2/ao_uo,ao_vo,ao_ui,ao_vi

!---fields used to feat the ocean/ice model-------------------
! hlatAO: downard latent turbulent heat flux (W.m-2)
! hsenAO: downard sensible turbulent heat flux (W.m-2)
! radsolAO: downard solar heat flux (W.m-2)
! radirAO: downard infrared heat flux (W.m-2)
! radtotAO: downard total non solar heat flux (W.m-2)
! DFlAO: latent flux derivative /temperature (W.m-2.K-1)
! DFsAO: sensible flux derivative /temperature (W.m-2.K-1)
! DFiAO: infrared flux derivative /temperature (W.m-2.K-1)
! DFtotAO: total non solar heat flux derivative /temperature (W.m-2.K-1)
! lp_accuAO,sp_accuAO,lprecipAO_t1,lprecipAO_t2,sprecipAO_t1,sprecipAO_t2
!                 (usefull to express lprecip and sprecip in kg.m-2.s-1)
! evapAO: evaporation on ice or water (kg.m-2.s-1)
! lprecipAO: liquid precipitation (kg.m-2.s-1)
! sprecipAO: solid precipitation (kg.m-2.s-1)
! upsnowAO: surface upward snow flux where sea ice (kg.m-2.s-1)
! UsrfAO: surface wind following the X-axis (m.s-1)
! VsrfAO: surface wind following the Y-axis (m.s-1) 
! TauxuAO: weighted surface downward X-axis stress on U-grid (Pa)
! TauyuAO: weighted surface downward Y-axis stress on U-grid (Pa)
! TauxvAO: weighted surface downward X-axis stress on V-grid (Pa)
! TauyvAO: weighted surface downward Y-axis stress on V-grid (Pa)
! TauxtAO: weighted surface downward X-axis stress on T-grid (Pa)
! TauytAO: weighted surface downward Y-axis stress on T-grid (Pa)



 
	real*8    hlatAO(mx,my,mw),hsenAO(mx,my,mw),radtotAO(mx,my,mw)
	real*8    radsolAO(mx,my,mw),radirAO(mx,my,mw)
	common /atmflx/hlatAO,hsenAO,radsolAO,radirAO,radtotAO
        real*8    DFlAO(mx,my,mw),DFsAO(mx,my,mw),DFiAO(mx,my,mw)           
        real*8                                  DFtotAO(mx,my,mw)
        common /derflx/DFlAO,DFsAO,DFiAO,DFtotAO	     


	real*8    lp_accuAO(mx,my),sp_accuAO(mx,my),lprecipAO_t1(mx,my)
        real*8  lprecipAO_t2(mx,my),sprecipAO_t1(mx,my)          
        real*8                            sprecipAO_t2(mx,my)
        common /watrat/lp_accuAO,sp_accuAO,lprecipAO_t1,lprecipAO_t2  
        common/watrat2/  sprecipAO_t1,sprecipAO_t2
	real*8    evapAO(mx,my),lprecipAO(mx,my),sprecipAO(mx,my)  
        real*8  ievpAO(mx,my)
	common /watflx/evapAO,lprecipAO,sprecipAO,ievpAO


	real*8    UsrfAO(mx,my),VsrfAO(mx,my),normdUAO
	common /atwind/UsrfAO,VsrfAO,normdUAO


	real*8    TauxuAO(mx,my,mw), TauyuAO(mx,my,mw)
	real*8    TauxvAO(mx,my,mw), TauyvAO(mx,my,mw)
        real*8    TauxtAO(mx,my,mw), TauytAO(mx,my,mw)
	common /stress/TauxuAO,TauyuAO,TauxvAO,TauyvAO,TauxtAO,TauytAO

	

!---usefull fields for coupling--------------------------------
! il_time_secs : time counter for coupling
! jtCpl : frequency of coupling (in time steps)
! iH2O : time counter usefull for precip calculation
! t_Cpl : time between 2 intocpl call (s)
! epsAO : emissivity 
! cpv : water vapor specific heat (J/kg/K)
! cpa : air specific heat (wet air) (J/kg/K)
! cpvir : [cpv/cp - 1] (usefull value for qsat) (-)
! R_Rv : [gas cste dry air=287.0] / [gas cste moist air=461.5]

!uuao: uair_dy (u component of the wind) on a regular grid
!vvao: vair_dy (v component of the wind) on a regular grid



	integer    il_time_secs,iH2O,info,il_time_exp0
        integer    jtCpl,tocken_AO
	common /intvar/il_time_secs,jtCpl,iH2O,info
        common /intval/tocken_AO,il_time_exp0


        real epsAO,cpv,cpvir,R_Rv,cpa,t_Cpl
        common /phycst/epsAO,cpv,cpa,cpvir,R_Rv,t_Cpl

       real uuao(mx,my), vvao(mx,my)
       common/windao/uuao,vvao

       logical coupling_ao
       real weightao(mx,my),weightao_sst(mx,my),weightao_sic(mx,my)
       real weightao_st(mx,my),weightao_al(mx,my),weightao_sit(mx,my)
       real weightao_snt(mx,my)
       common/coupl1/coupling_ao,weightao,weightao_sst,weightao_sic
       common/coupl2/weightao_st,weightao_al,weightao_sit
       common/coupl3/weightao_snt
       
       character*300 fileao

     
     
! il_time_secs : time counter for coupling
! jtCpl : frequency of coupling (in time steps)
! iH2O : time counter usefull for precip calculation
! t_Cpl : time between 2 intocpl call (s)
! epsAO : emissivity 
! cpv : water vapor specific heat (J/kg/K)
! cpa : air specific heat (wet air) (J/kg/K)
! cpvir : [cpv/cp - 1] (usefull value for qsat) (-)
! R_Rv : [gas cste dry air=287.0] / [gas cste moist air=461.5]

!---Sea Ice/Snow albedo for SISVAT albAOsisv is defined in MARySV.inc--------------------------------
