 
 
      subroutine MARgx11mx1my
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           05-07-2004  MAR |
C |   SubRoutine MARgx11mx1my performs Gaussian Elimination along x-Dir    |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                                                               |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  ix
 
      data ix /0/
 
      k    =   1
 
 
C +--Forward  Sweep
C +  ==============
 
      IF (ix.ne.1)                                                  THEN
          DO j= 1,my
            WKxyz5(1,j,k)= WKxyz2(1,j,k)
            WKxyz6(1,j,k)=-WKxyz1(1,j,k)/WKxyz5(1,j,k)
          END DO
          DO i=ip11,mx
          DO j=   1,my
            WKxyz5(i,j,k)= WKxyz3(i,j,k)*WKxyz6(i-1,j,k)+WKxyz2(i,j,k)
            WKxyz6(i,j,k)=-WKxyz1(i,j,k)/WKxyz5(i  ,j,k)
          END DO
          END DO
      END IF
 
          DO j= 1,my
            WKxyz7(1,j,k)= WKxyz4(1,j,k)/WKxyz5(1,j,k)
          END DO
 
          DO i=ip11,mx
          DO j=   1,my
            WKxyz7(i,j,k)=(WKxyz4(i,j,k)-WKxyz3(i,j,k)*WKxyz7(i-1,j,k))
     .                    /WKxyz5(i,j,k)
          END DO
          END DO
 
 
C +--Backward Sweep
C +  ==============
 
          DO i=mx1,1,-1
          DO j=  1,my
            WKxyz7(i,j,k)=WKxyz6(i,j,k)*WKxyz7(i+1,j,k)+WKxyz7(i,j,k)
          END DO
          END DO
 
      return
      end
 
 
      subroutine MARgx_1mx1my
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           05-07-2004  MAR |
C |   SubRoutine MARgx_1mx1my performs Gaussian Elimination along x-Dir    |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                                                               |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  ix
 
      data ix /0/
 
 
C +--Forward  Sweep
C +  ==============
 
      IF (ix.ne.1)                                                  THEN
          DO k= 1,mz
          DO j= 1,my
            WKxyz5(1,j,k)= WKxyz2(1,j,k)
            WKxyz6(1,j,k)=-WKxyz1(1,j,k)/WKxyz5(1,j,k)
          END DO
          END DO
        DO   i=ip11,mx
          DO k=   1,mz
          DO j=   1,my
            WKxyz5(i,j,k)= WKxyz3(i,j,k)*WKxyz6(i-1,j,k)+WKxyz2(i,j,k)
            WKxyz6(i,j,k)=-WKxyz1(i,j,k)/WKxyz5(i  ,j,k)
          END DO
          END DO
        END DO
      END IF
 
          DO k= 1,mz
          DO j= 1,my
            WKxyz7(1,j,k)= WKxyz4(1,j,k)/WKxyz5(1,j,k)
          END DO
          END DO
 
        DO   i=ip11,mx
          DO k=   1,mz
          DO j=   1,my
            WKxyz7(i,j,k)=(WKxyz4(i,j,k)-WKxyz3(i,j,k)*WKxyz7(i-1,j,k))
     .                    /WKxyz5(i,j,k)
          END DO
          END DO
        END DO
 
 
C +--Backward Sweep
C +  ==============
 
        DO   i=mx1,1,-1
          DO k=  1,mz
          DO j=  1,my
            WKxyz7(i,j,k)=WKxyz6(i,j,k)*WKxyz7(i+1,j,k)+WKxyz7(i,j,k)
          END DO
          END DO
        END DO
 
      return
      end
 
 
      subroutine MARgy11mx1my
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           05-07-2004  MAR |
C |   SubRoutine MARgy11mx1my performs Gaussian Elimination along y-Dir.   |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                                                               |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  ix
 
      data ix /0/
 
      k    =   1
 
 
C +--Forward  Sweep
C +  ==============
 
      IF (ix.ne.1)                                                  THEN
          DO i= 1,mx
            WKxyz5(i,1,k)= WKxyz2(i,1,k)
            WKxyz6(i,1,k)=-WKxyz1(i,1,k)/WKxyz5(i,1,k)
          END DO
          DO j=jp11,my
          DO i=   1,mx
            WKxyz5(i,j,k)= WKxyz3(i,j,k)*WKxyz6(i,j-1,k)+WKxyz2(i,j,k)
            WKxyz6(i,j,k)=-WKxyz1(i,j,k)/WKxyz5(i,j  ,k)
          END DO
          END DO
      END IF
 
          DO i= 1,mx
            WKxyz7(i,1,k)= WKxyz4(i,1,k)/WKxyz5(i,1,k)
          END DO
 
          DO j=jp11,my
          DO i=   1,mx
            WKxyz7(i,j,k)=(WKxyz4(i,j,k)-WKxyz3(i,j,k)*WKxyz7(i,j-1,k))
     .                    /WKxyz5(i,j,k)
          END DO
          END DO
 
 
C +--Backward Sweep
C +  ==============
 
          DO j=my1,1,-1
          DO i=  1,mx
            WKxyz7(i,j,k)=WKxyz6(i,j,k)*WKxyz7(i,j+1,k)+WKxyz7(i,j,k)
          END DO
          END DO
 
      return
      end
 
 
      subroutine MARgy_1mx1my
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           05-07-2004  MAR |
C |   SubRoutine MARgy_1mx1my performs Gaussian Elimination along y-Dir.   |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                                                               |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  ix
 
      data ix /0/
 
 
C +--Forward  Sweep
C +  ==============
 
      IF (ix.ne.1)                                                  THEN
        DO   k= 1,mz
          DO i= 1,mx
            WKxyz5(i,1,k)= WKxyz2(i,1,k)
            WKxyz6(i,1,k)=-WKxyz1(i,1,k)/WKxyz5(i,1,k)
          END DO
        END DO
        DO   j=jp11,my
          DO k=   1,mz
          DO i=   1,mx
            WKxyz5(i,j,k)= WKxyz3(i,j,k)*WKxyz6(i,j-1,k)+WKxyz2(i,j,k)
            WKxyz6(i,j,k)=-WKxyz1(i,j,k)/WKxyz5(i,j  ,k)
          END DO
          END DO
        END DO
      END IF
 
          DO k= 1,mz
          DO i= 1,mx
            WKxyz7(i,1,k)= WKxyz4(i,1,k)/WKxyz5(i,1,k)
          END DO
          END DO
 
        DO   j=jp11,my
          DO k=   1,mz
          DO i=   1,mx
            WKxyz7(i,j,k)=(WKxyz4(i,j,k)-WKxyz3(i,j,k)*WKxyz7(i,j-1,k))
     .                    /WKxyz5(i,j,k)
          END DO
          END DO
        END DO
 
 
C +--Backward Sweep
C +  ==============
 
        DO j=my1,1,-1
          DO k=  1,mz
          DO i=  1,mx
            WKxyz7(i,j,k)=WKxyz6(i,j,k)*WKxyz7(i,j+1,k)+WKxyz7(i,j,k)
          END DO
          END DO
        END DO
 
      return
      end
 
 
 
 
      subroutine MARgz_1mx1my(k1,k2,jj)
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           08-09-2017  MAR |
C |   SubRoutine MARgz_1mx1my performs Gaussian Elimination along s-Dir.   |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                      k1,k2: k Loops Limits                    |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  k1,k2,ii,jj,kk
 
 
C +--Forward  Sweep
C +  ==============
 
c       DO j=1,my
        DO ii=1,mx
          WKxyz5(ii,jj,k1) = WKxyz2(ii,jj,k1)
          WKxyz6(ii,jj,k1) =-WKxyz1(ii,jj,k1) /WKxyz5(ii,jj,k1)
        END DO
c       END DO
      DO   kk=kp1(k1),k2
c       DO j=1,my
        DO ii=1,mx
          WKxyz5(ii,jj,kk) = WKxyz3(ii,jj,kk)   *WKxyz6(ii,jj,kk-1)
     .                     + WKxyz2(ii,jj,kk)
          WKxyz6(ii,jj,kk) =-WKxyz1(ii,jj,kk)   /WKxyz5(ii,jj,kk)
        END DO
c       END DO
      END DO
c       DO j=1,my
        DO ii=1,mx
          WKxyz7(ii,jj,k1)= WKxyz4(ii,jj,k1)  /WKxyz5(ii,jj,k1)
        END DO
c       END DO
      DO   kk=kp1(k1),k2
c       DO j=1,my
        DO ii=1,mx
          WKxyz7(ii,jj,kk) =(WKxyz4(ii,jj,kk)   -WKxyz3(ii,jj,kk)
     .                      *WKxyz7(ii,jj,kk-1))/WKxyz5(ii,jj,kk)
        END DO
c       END DO
      END DO
 
 
C +--Backward Sweep
C +  ==============
 
      DO kk=km1(k2),k1,-1
c       DO j=1,my
        DO ii=1,mx
          WKxyz7(ii,jj,kk) =  WKxyz6(ii,jj,kk) *WKxyz7(ii,jj,kk+1)
     .                     +  WKxyz7(ii,jj,kk)
        END DO
c       END DO
      END DO
 
      return
      end
 
 
      subroutine MARgz_2mx1y1_mp(kk1,kk2,jj)
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS                                           03-07-2004  MAR |
C |   SubRoutine MARgz_2mx1y1 performs Gaussian Elimination along s-Dir.   |
C |    (e.g. Pielke (1984), pp.302--303)                                   |
C |    (needed to solve the implicit scheme developped for filtering)      |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT:                      k1,k2: k Loops Limits                    |
C |   ^^^^^                                                                |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_WK.inc'
 
      integer  kk1,kk2,jj,ii,kk
 
C +--Forward  Sweep
C +  ==============
 
c       DO jj=jjp11,my1
        DO ii=ip11,mx1
          WKxyz5(ii,jj,kk1) = WKxyz2(ii,jj,kk1)
          WKxyz6(ii,jj,kk1) =-WKxyz1(ii,jj,kk1) /WKxyz5(ii,jj,kk1)
        END DO
c       END DO
      DO   kk=kp1(kk1),kk2
c       DO jj=jjp11,my1
        DO ii=ip11,mx1
          WKxyz5(ii,jj,kk)= WKxyz3(ii,jj,kk)*WKxyz6(ii,jj,kk-1)
     .                     +WKxyz2(ii,jj,kk)
          WKxyz6(ii,jj,kk)=-WKxyz1(ii,jj,kk)/WKxyz5(ii,jj,kk)
        END DO
c       END DO
      END DO
c       DO jj=jjp11,my1
        DO ii=ip11,mx1
          WKxyz7(ii,jj,kk1)= WKxyz4(ii,jj,kk1)  /WKxyz5(ii,jj,kk1)
        END DO
c       END DO
      DO   kk=kp1(kk1),kk2
c       DO jj=jjp11,my1
        DO ii=ip11,mx1
          WKxyz7(ii,jj,kk) =(WKxyz4(ii,jj,kk)   -WKxyz3(ii,jj,kk)
     .                     * WKxyz7(ii,jj,kk-1))/WKxyz5(ii,jj,kk)
        END DO
c       END DO
      END DO
 
 
C +--Backkward Sweep
C +  ==============
 
      DO kk=km1(kk2),kk1,-1
c       DO jj=jjp11,my1
        DO ii=ip11,mx1
          WKxyz7(ii,jj,kk)=WKxyz6(ii,jj,kk)*WKxyz7(ii,jj,kk+1)
     .                    +WKxyz7(ii,jj,kk)
        END DO
        END DO
c      END DO
 
      return
      end
